/** 
 * Class: OpenLayers.Layer.CartoDB
 * Esta clase que extiende de Grid permite obtener capas de cartoDB, con sus estilos y consutas se ha tomado XYZ como 
 * referencia para crearla
 * 
 * Inherits from:
 *  - <OpenLayers.Layer.Grid>
 */

/**
 * @requires OpenLayers/Layer/Grid.js
 */

OpenLayers.Layer.CartoDB = OpenLayers.Class(OpenLayers.Layer.Grid, {
    
    /**
    *
    * Variables que guardan el estilo y la consulta SQL para enviarla en cada petición de tile
    *
    */
    style: '',
    sql:'',

    /**
     * APIProperty: isBaseLayer
     * Por defecto si, indica si es capa base. 
     */
    isBaseLayer: true,
    
    /**
     * APIProperty: sphericalMecator
     * Por defecto no, indica si utilizamos proyección esférica mercator.
     */
    sphericalMercator: false,

    
    /**
     * APIProperty: serverResolutions
     * {Array} Resoluciones de la capa, por defecto las calcula automáticamente.
     */
    serverResolutions: null,

    /**
     * APIProperty: setSQL
     * {String} Nos permite fijar la consulta que se añadirá a cada petición de cada imagen.
     */
    setSQL: function(sql){
        this.sql = sql;
    },
    
    /**
     * APIProperty: setSQL
     * Nos permite obtener la consulta previamente fijada que se añadirá a cada petición de cada imagen.
     */
    getSQL: function(){
        return this.sql;
    },
    
    /**
     * APIProperty: setSQL
     * {String} Nos permite fijar el estilo de la capa, es una cadena con el estandar CartoCSS sin saltos de línea.
     */
    setStyle: function(style){
        this.style = style;
    },

    /**
     * Constructor: OpenLayers.Layer.CartoDB
     *
     * Parameters:
     * name - {String}
     * url - {String}
     * options - {Object} Hash de opciones
     * Extendido de <OpenLayers.Layer.XYZ>
     */
    initialize: function(name, url, options) {
        if (options && options.sphericalMercator || this.sphericalMercator) {
            options = OpenLayers.Util.extend({
                projection: "EPSG:900913",
                numZoomLevels: 19
            }, options);
        }
        OpenLayers.Layer.Grid.prototype.initialize.apply(this, [
            name || this.name, url || this.url, {}, options
        ]);
    },
    
    /**
     * APIMethod: clone
     * Crea un clon de la capa
     *
     * Parameters:
     * obj - {Object} Ha sido usado
     * 
     * Returns:
     * {<OpenLayers.Layer.CartoDB>} una copia exacta del objeto OpenLayers.Layer.CartoDB
     * Extendido de <OpenLayers.Layer.XYZ>
     */
    clone: function (obj) {
        
        if (obj == null) {
            obj = new OpenLayers.Layer.CartoDB(this.name,
                                            this.url,
                                            this.getOptions());
        }
        obj = OpenLayers.Layer.Grid.prototype.clone.apply(this, [obj]);
        return obj;
    },    

    /**
     * Method: getURL
     *
     * Parameters:
     * bounds - {<OpenLayers.Bounds>}
     *
     * Returns:
     * {String} Limites
     */
    getURL: function (bounds) {
        var xyz = this.getXYZ(bounds);
        var url = this.url;
        if (OpenLayers.Util.isArray(url)) {
            var s = '' + xyz.x + xyz.y + xyz.z;
            url = this.selectUrl(s, url);
        }
        //Añadimos un random para refrescar cada vez que tengamos un cambio. esto hace que el navegador no cachee 
        //las imágenes
        var res =  OpenLayers.String.format(url, xyz) + "?r=" + Math.random();
        //Añadimos la consulta y el estilo.
        if(this.sql!=""){ res = res + "&sql=" + encodeURIComponent(this.sql);}
        if(this.style!=""){ res = res + "&style=" + encodeURIComponent(this.style);}
        return res;
    },
    
    /**
     * Method: getXYZ
     * Calcula la X Y y Z para los limites pasados.
     *
     * Parameters:
     * bounds - {<OpenLayers.Bounds>}
     *
     * Returns:
     * {Object} - un objecto con x, y y z.
     * Extendido de <OpenLayers.Layer.XYZ>
     */
    getXYZ: function(bounds) {
        var res = this.getServerResolution();
        var x = Math.round((bounds.left - this.maxExtent.left) /
            (res * this.tileSize.w));
        var y = Math.round((this.maxExtent.top - bounds.top) /
            (res * this.tileSize.h));
        var z = this.getServerZoom();

        if (this.wrapDateLine) {
            var limit = Math.pow(2, z);
            x = ((x % limit) + limit) % limit;
        }

        return {'x': x, 'y': y, 'z': z};
    },
    
    /* APIMethod: setMap
     * Permite obtener el mapa al que se vincula esta capa
     * 
     * Parameters:
     * map - {<OpenLayers.Map>}
     * Extendido de <OpenLayers.Layer.XYZ>
     */
    setMap: function(map) {
        OpenLayers.Layer.Grid.prototype.setMap.apply(this, arguments);
        if (!this.tileOrigin) { 
            this.tileOrigin = new OpenLayers.LonLat(this.maxExtent.left,
                                                this.maxExtent.bottom);
        }                                       
    },

    CLASS_NAME: "OpenLayers.Layer.CartoDB"
});
